package com.hortonworks.spark.sql.hive.llap.writers;

import java.io.IOException;

import com.hortonworks.spark.sql.hive.llap.util.SerializableHadoopConfiguration;
import com.hortonworks.spark.sql.hive.llap.util.SparkToHiveRecordMapper;
import com.hortonworks.spark.sql.hive.llap.writers.HiveWarehouseDataWriterHelper.FileFormat;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.spark.sql.catalyst.InternalRow;
import org.apache.spark.sql.execution.datasources.BasicWriteJobStatsTracker;
import org.apache.spark.sql.sources.v2.writer.DataWriter;
import org.apache.spark.sql.sources.v2.writer.DataWriterFactory;
import org.apache.spark.sql.types.StructType;

import java.util.Map;

public class HiveWarehouseDataWriterFactory implements DataWriterFactory<InternalRow> {

  protected String jobId;
  protected StructType schema;
  private FileFormat fileFormat;
  private Path path;
  private Map<String, String> options;
  private SerializableHadoopConfiguration conf;
  private SparkToHiveRecordMapper sparkToHiveRecordMapper;
  private BasicWriteJobStatsTracker basicWriteJobStatsTracker;

  public HiveWarehouseDataWriterFactory(String jobId, StructType schema, FileFormat fileFormat,
                                        Path path, Map<String, String> options, SerializableHadoopConfiguration conf,
                                        SparkToHiveRecordMapper sparkToHiveRecordMapper,
                                        BasicWriteJobStatsTracker basicWriteJobStatsTracker) {
    this.jobId = jobId;
    this.schema = schema;
    this.fileFormat = fileFormat;
    this.path = path;
    this.options = options;
    this.conf = conf;
    this.sparkToHiveRecordMapper = sparkToHiveRecordMapper;
    this.basicWriteJobStatsTracker = basicWriteJobStatsTracker;

  }

  @Override
  public DataWriter<InternalRow> createDataWriter(int partitionId, long taskId, long epochId) {
    Path filePath = new Path(this.path, String.format("%s_%s_%s_%s", jobId, partitionId, taskId, epochId));
    FileSystem fs = null;
    try {
      fs = filePath.getFileSystem(conf.get());
    } catch (IOException e) {
      e.printStackTrace();
    }
    return getDataWriter(conf.get(), jobId, schema, fileFormat, options, partitionId, taskId, epochId,
            fs, filePath, sparkToHiveRecordMapper, basicWriteJobStatsTracker);
  }

  protected DataWriter<InternalRow> getDataWriter(Configuration conf, String jobId, StructType schema, FileFormat fileFormat,
                                                  Map<String, String> options, int partitionId, long taskId, long epochId,
                                                  FileSystem fs, Path filePath, SparkToHiveRecordMapper sparkToHiveRecordMapper,
                                                  BasicWriteJobStatsTracker basicWriteJobStatsTracker) {
    return new HiveWarehouseDataWriter(conf, jobId, schema, fileFormat, options, partitionId, taskId, epochId, fs, filePath,
            sparkToHiveRecordMapper, basicWriteJobStatsTracker);
  }
}